#------------------------------------------------------------------------------
#
#  Copyright (c) 2014-2015, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#------------------------------------------------------------------------------
""" RangeEditor replacement.
"""

import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))  # noqa

from traits.api import HasTraits, Range
from traitsui.api import Item, View

from qt_binder.api import Bound, TraitsUI
from qt_binder.qt import QtGui
from qt_binder.raw_widgets import FormLayout
from qt_binder.widgets import RangeSlider


class Model(HasTraits):
    low_bound = Range(low=0, high=40)
    high_bound = Range(low=60, high=100)
    x = Range(low='low_bound', high='high_bound')

    def default_traits_view(self):
        traits_view = View(
            Bound(
                FormLayout(
                    (u'Low:',
                     RangeSlider(id='low', range=(0, 40))),
                    (u'High:',
                     RangeSlider(id='high', range=(60, 100))),
                    (u'RangeEditor:',
                     TraitsUI(Item('x'))),
                    fieldGrowthPolicy=QtGui.QFormLayout.ExpandingFieldsGrow,
                ),
                'low.value := object.low_bound',
                'high.value := object.high_bound',
                # Make the labels align nicely by fixing their widths using
                # stylesheets.
                stylesheet=(u'*[binder_class="RangeSlider"] > .QLabel '
                            u'{min-width: 30px; max-width: 30px;}'),
            ),
            resizable=True,
            title=u'Traits UI Editor',
        )
        return traits_view


def main():
    m = Model()
    m.configure_traits()

if __name__ == '__main__':
    main()



from contextlib import contextmanager

from pyface.ui.qt4.util.gui_test_assistant import GuiTestAssistant
from traits.trait_notifiers import pop_exception_handler, \
    push_exception_handler


class BaseTestWithGui(GuiTestAssistant):
    """ Base class for testing Binders.

    Starts a Qt event loop and reraise exceptions during traits event handling.
    """

    def setUp(self):
        GuiTestAssistant.setUp(self)
        push_exception_handler(reraise_exceptions=True)

    def tearDown(self):
        pop_exception_handler()
        GuiTestAssistant.tearDown(self)

    @contextmanager
    def constructed(self, binder):
        """ Take care of the lifecycle of a binder.

        Construct and configure a binder on entry, and dispose it at exit.
        """
        binder.construct()
        try:
            binder.configure()
            yield binder
        finally:
            binder.dispose()



#------------------------------------------------------------------------------
#
#  Copyright (c) 2014-2015, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#------------------------------------------------------------------------------

# Add a NullHandler so loggers don't complain when they get used.
import logging


class NullHandler(logging.Handler):

    def handle(self, record):
        pass

    def emit(self, record):
        pass

    def createLock(self):
        self.lock = None

logger = logging.getLogger(__name__)
logger.addHandler(NullHandler())

del logging, logger, NullHandler



#------------------------------------------------------------------------------
#
#  Copyright (c) 2014-2015, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#------------------------------------------------------------------------------

import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))  # noqa

from traits.api import HasTraits
from traitsui.api import View

from qt_binder.api import Bound
from qt_binder.qt.QtCore import Qt
from qt_binder.raw_widgets import BasicGridLayout, CheckBox, GroupBox, \
    LineEdit, PushButton, SpanGridLayout, VBoxLayout


class Model(HasTraits):
    """ We're just showing off layout, so no model traits.
    """

    def default_traits_view(self):
        traits_view = View(
            Bound(
                VBoxLayout(
                    GroupBox(
                        BasicGridLayout(
                            [u'Always On:', None, LineEdit()],
                            [u'Enableable:',
                             CheckBox(id='field_checkbox'),
                             LineEdit(id='field')],
                            [u'Left:',
                             CheckBox(id='left_checkbox'),
                             (LineEdit(id='left'), Qt.AlignLeft)],
                            [u'Right:',
                             CheckBox(id='right_checkbox'),
                             (LineEdit(id='right'), Qt.AlignRight)],
                        ),
                        title=u'Basic Grid Layout',
                    ),
                    GroupBox(
                        SpanGridLayout(
                            (PushButton(text=u'(0, 0)'), 0, 0),
                            (PushButton(text=u'(0, 1) - (0, 2)'), 0, 1, 1, 2),
                            (PushButton(text=u'(1, 0) - (1, 1)'), 1, 0, 1, 2),
                            (PushButton(
                                text=u'(1, 2)\N{RIGHTWARDS ARROW TO BAR}'),
                             1, 2, Qt.AlignRight),
                            (PushButton(text=u'(2, 0)'), 2, 0),
                            (PushButton(text=u'(2, 2)'), 2, 2),
                        ),
                        title=u'Span Grid Layout',
                    ),
                ),
                'field.enabled << field_checkbox.checked',
                'left.enabled << left_checkbox.checked',
                'right.enabled << right_checkbox.checked',
            ),
            resizable=True,
            title=u'Grid Layouts',
        )
        return traits_view


def main():
    Model().configure_traits()

if __name__ == '__main__':
    main()



#------------------------------------------------------------------------------
#
#  Copyright (c) 2014-2015, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#------------------------------------------------------------------------------

from abc import ABCMeta, abstractmethod
from collections import deque
import parser
import re
import symbol
import token

import six

from traits.trait_base import xgetattr, xsetattr

from .binder import Binder


class AnyString(object):
    """ Compare equal to any string type.
    """
    def __eq__(self, other):
        return isinstance(other, six.string_types)

    def __ne__(self, other):
        return not (self == other)


def yield_subtrees(expr):
    """ Preorder traversal of all subtrees in an expression.
    """
    root = parser.expr(expr).totuple()
    stack = deque([root])
    while stack:
        node = stack.popleft()
        children = [child for child in node[1:] if isinstance(child, tuple)]
        stack.extend(children)
        yield node


def find_ext_attrs(expr):
    """ Find all dotted references in the expression.
    """
    ext_attrs = []
    for subtree in yield_subtrees(expr):
        if subtree[1] == (symbol.atom, (token.NAME, AnyString())):
            parts = [subtree[1][1][1]]
            for trailer in subtree[2:]:
                if trailer == (symbol.trailer,
                               (token.DOT, '.'),
                               (token.NAME, AnyString())):
                    parts.append(trailer[2][1])
                else:
                    break
            if len(parts) > 1:
                ext_attrs.append('.'.join(parts))
    return ext_attrs


class _TraitModified(object):
    """ Expose a well-formed trait change handler function with extra data.

    This is a workaround to avoid some problems with defining change handler
    functions inside of other functions. The ``handler()`` method is a proper
    trait change handler.
    """
    def __init__(self, obj, xattr):
        self.obj = obj
        self.xattr = xattr
        self._in_handler = False

    def handler(self, new):
        if not self._in_handler:
            self._in_handler = True
            try:
                xsetattr(self.obj, self.xattr, new)
            finally:
                self._in_handler = False


class _EvaluateExpression(_TraitModified):

    def __init__(self, obj, xattr, context, expression):
        super(_EvaluateExpression, self).__init__(obj, xattr)
        self.context = context
        self.expression = expression

    def handler(self):
        if not self._in_handler:
            self._in_handler = True
            try:
                value = eval(self.expression, self.context)
                xsetattr(self.obj, self.xattr, value)
            finally:
                self._in_handler = False


class Binding(object):
    """ Interface for a single binding pair.
    """
    __metaclass__ = ABCMeta

    _op_regex = re.compile(r'\s*(=|>>|<<|:=)\s*')

    def __init__(self, left, right):
        self.left = left
        self.right = right

    def __repr__(self):
        return '{0.__name__}({1.left!r}, {1.right!r})'.format(type(self), self)

    def __eq__(self, other):
        if type(other) is not type(self):
            return False
        return (self.left, self.right) == (other.left, other.right)

    def __ne__(self, other):
        return not (self == other)

    def __hash__(self):
        return hash((type(self), self.left, self.right))

    @classmethod
    def parse(cls, obj):
        """ Parse a binding expression into the right :class:`~.Binding`
        subclass.
        """
        if isinstance(obj, six.string_types):
            left, op, right = cls._op_regex.split(obj, 1)
            return {
                '=': SetOnceTo,
                ':=': SyncedWith,
                '>>': PushedTo,
                '<<': PulledFrom,
            }[op](left, right)
        elif isinstance(obj, Binding):
            return obj
        else:
            raise TypeError("Expected Binding, or binding string; "
                            "got {0!r}".format(obj))

    @abstractmethod
    def bind(self, binder, context):
        """ Perform the binding and store the information needed to undo it.
        """
        raise NotImplementedError()

    @abstractmethod
    def unbind(self):
        """ Undo the binding.
        """
        raise NotImplementedError()

    def _normalize_binder_trait(self, binder, binder_trait, context):
        """ Normalize the Binder and binder trait.
        """
        if '.' in binder_trait:
            head, tail = binder_trait.split('.', 1)
            if isinstance(context.get(head, None), Binder):
                binder = context[head]
                binder_trait = tail
        return binder, binder_trait


class Factory(Binding):
    """ Call the factory to initialize a value.

    The right item of the pair is a callable that will be called once on
    initialization to provide a value for the destination trait.
    """

    def bind(self, binder, context):
        the_binder, binder_trait = self._normalize_binder_trait(
            binder, self.left, context)
        value = self.right()
        xsetattr(the_binder, binder_trait, value)

    def unbind(self):
        pass


class SetOnceTo(Binding):
    """ Evaluate values once.

    The right item of the pair is a string that will be evaluated in the Traits
    UI context once on initialization.

    Mnemonic: ``binder_trait is set once to expression``
    """
    def bind(self, binder, context):
        expression = self.right
        the_binder, binder_trait = self._normalize_binder_trait(
            binder, self.left, context)
        value = eval(expression, context)
        xsetattr(the_binder, binder_trait, value)

    def unbind(self):
        pass

    def __str__(self):
        return '{0.left} = {0.right}'.format(self)


class PulledFrom(Binding):
    """ Listen to traits in the context.

    The right item of each pair is a string representing the extended trait to
    listen to. The first part of this string should be a key into the Traits UI
    context; e.g. to listen to the ``foo`` trait on the model object, use
    ``'object.foo'``. When the ``foo`` trait on the model object fires a trait
    change notification, the ``Binder`` trait will be assigned. The reverse is
    not true: see :class:`~.PushedTo` and :class:`~.SyncedWith` for that
    functionality.

    Mnemonic: ``binder_trait is pulled from context_trait``
    """
    # FIXME: Allow users to explicitly specify a `depends_on` list. This would
    # let users avoid problems with too many dots.
    def bind(self, binder, context):
        the_binder, binder_trait = self._normalize_binder_trait(
            binder, self.left, context)
        rhs = self.right.strip()
        ext_traits = find_ext_attrs(rhs)
        if ext_traits == [rhs]:
            # Simple case of one attribute.
            context_name, xattr = rhs.split('.', 1)
            context_obj = context[context_name]

            handler = _TraitModified(the_binder, binder_trait).handler
            # FIXME: Only check as far down as are HasTraits objects available.
            # We would like to be able to include references to methods on
            # attributes of HasTraits classes.
            # Unfortunately, a valid use case is where a leading object in
            # a true trait chain is None.
            context_obj.on_trait_change(handler, xattr)
            self.pull_handler_data = [(context_obj, handler, xattr)]
            # FIXME: do a better check for an event trait
            try:
                xsetattr(the_binder, binder_trait,
                         xgetattr(context_obj, xattr))
            except AttributeError as e:
                if 'event' not in str(e):
                    raise
        elif ext_traits == []:
            msg = "No traits found in expression: {0!r}".format(rhs)
            raise ValueError(msg)
        else:
            # Expression.
            self.pull_handler_data = []
            handler = _EvaluateExpression(the_binder, binder_trait,
                                          context, rhs).handler
            for ext_trait in ext_traits:
                context_name, xattr = ext_trait.split('.', 1)
                if context_name not in context:
                    # Assume it's a builtin.
                    continue
                context_obj = context[context_name]
                context_obj.on_trait_change(handler, xattr)
                self.pull_handler_data.append((context_obj, handler, xattr))
            # Call the handler once to evaluate and set the value initially.
            handler()

    def unbind(self):
        for context_obj, handler, xattr in self.pull_handler_data:
            context_obj.on_trait_change(handler, xattr, remove=True)

    def __str__(self):
        return '{0.left} << {0.right}'.format(self)


class PushedTo(Binding):
    """ Send trait updates from the ``Binder`` to the model.

    The right item of each pair is a string representing the extended trait to
    assign the value to. The first part of this string should be a key into the
    Traits UI context; e.g. to send to the ``foo`` trait on the model object,
    use ``'object.foo'``. When a change notification for ``binder_trait`` is
    fired, ``object.foo`` will be assigned the sent object. The reverse is not
    true: see :class:`~.PulledFrom` and :class:`~.SyncedWith` for that
    functionality.

    Mnemonic: ``binder_trait is sent to context_trait``
    """
    def bind(self, binder, context):
        ext_trait = self.right
        the_binder, binder_trait = self._normalize_binder_trait(
            binder, self.left, context)
        context_name, xattr = ext_trait.split('.', 1)
        context_obj = context[context_name]

        handler = _TraitModified(context_obj, xattr).handler
        the_binder.on_trait_change(handler, binder_trait)
        self.pushed_handler_data = (the_binder, handler, binder_trait)

    def unbind(self):
        the_binder, handler, binder_trait = self.pushed_handler_data
        the_binder.on_trait_change(handler, binder_trait, remove=True)

    def __str__(self):
        return '{0.left} >> {0.right}'.format(self)


class SyncedWith(PulledFrom, PushedTo):
    """ Bidirectionally synchronize a ``Binder`` trait and a model trait.

    The right item of each pair is a string representing the extended trait to
    synchronize the binder trait with. The first part of this string should be
    a key into the Traits UI context; e.g. to synchronize with the ``foo``
    trait on the model object, use ``'object.foo'``. When a change notification
    for either trait is sent, the value will be assigned to the other. See
    :class:`~.PulledFrom` and :class:`~.PushedTo` for unidirectional
    synchronization.

    Mnemonic: ``binder_trait is synced with context_trait``
    """
    def bind(self, binder, context):
        PulledFrom.bind(self, binder, context)
        PushedTo.bind(self, binder, context)

    def unbind(self):
        PushedTo.unbind(self)
        PulledFrom.unbind(self)

    def __str__(self):
        return '{0.left} := {0.right}'.format(self)



#------------------------------------------------------------------------------
# Copyright (c) 2015, Enthought Inc
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD license

#
# Author: Enthought Inc
# Description: Qt API selector. Can be used to switch between pyQt and PySide
#------------------------------------------------------------------------------

import os


def prepare_pyqt4():
    # Set PySide compatible APIs.
    import sip
    sip.setapi('QDate', 2)
    sip.setapi('QDateTime', 2)
    sip.setapi('QString', 2)
    sip.setapi('QTextStream', 2)
    sip.setapi('QTime', 2)
    sip.setapi('QUrl', 2)
    sip.setapi('QVariant', 2)

qt_api = os.environ.get('QT_API')

if qt_api is None:
    try:
        import PySide  # noqa
        qt_api = 'pyside'
    except ImportError:
        try:
            prepare_pyqt4()
            import PyQt4  # noqa
            qt_api = 'pyqt'
        except ImportError:
            raise ImportError('Cannot import PySide or PyQt4')

elif qt_api == 'pyqt':
    prepare_pyqt4()

elif qt_api != 'pyside':
    msg = "Invalid Qt API %r, valid values are: 'pyqt' or 'pyside'" % qt_api
    raise RuntimeError(msg)



#------------------------------------------------------------------------------
#
#  Copyright (c) 2014-2015, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#------------------------------------------------------------------------------

from collections import Counter, deque
import keyword
import weakref

import six

from traits.api import HasStrictTraits, Instance, List, Property, Str, \
    TraitType, Undefined

from .constants import DELAYED_CONNECTION, DELAYED_SETATTR, \
    EXISTING_INSTANCE_TRAIT, EXISTING_NOTIFIERS, FORCE_INSTANCE_TRAIT, \
    FORCE_NOTIFIERS
from .loopback_guard import LoopbackGuard
from .qt import QtCore


NULL_VARIANT_VALUES = {
    'QString': u'',
}


def _slot_name(name):
    return '_{}_property_changed'.format(name)


def _slot_for(ref, name):
    def slot(*args):
        obj = ref()
        if obj is not None:
            if len(args) == 1:
                args = args[0]
            obj.trait_property_changed(name, Undefined, args)
    slot.__name__ = _slot_name(name)
    return slot


def _guard_against_null_variant(value):
    """ Convert PyQt4's QPyNullVariant to a reasonable value.
    """
    if type(value).__name__ == 'QPyNullVariant':
        return NULL_VARIANT_VALUES.get(value.typeName(), None)
    return value


def _python_name_for_qt_name(qname):
    """ Convert forbidden Qt names to valid Python names, following PySide/PyQt
    rules.

    E.g. 'raise' -> 'raise_'
    """
    if keyword.iskeyword(qname):
        qname += '_'
    return qname


class QtTrait(TraitType):
    """ Base class for Qt proxy traits on :class:`~.Binder` classes.

    Each subclass should override :meth:`get` and :meth:`set`. All
    :class:`~.QtTrait` subclasses are property-like traits.

    If there is a Qt ``Signal`` that should be connected to to propagate
    notifications, assign it to the ``signal`` attribute. The Qt ``Signal``
    will only be connected to when a Traits listener is attached to this trait.
    """

    def __init__(self, *args, **metadata):
        self.signal = None
        super(QtTrait, self).__init__(transient=True, **metadata)

    def get(self, object, name):
        """ Get the value of this trait.
        """
        raise NotImplementedError

    def set(self, object, name, value):
        """ Set the value of this trait and notify listeners.
        """
        raise NotImplementedError

    def connect_signal(self, object, name):
        """ Connect to the Qt signal, if any.
        """
        if self.signal is None:
            # No signal to connect to.
            return
        qobj = object.qobj
        slot = _slot_for(weakref.ref(object), name)
        object.__dict__[slot.__name__] = slot
        # Use the classmethod form for PyQt4 compatibility.
        QtCore.QObject.connect(qobj, self.signal, slot)

    def disconnect_signal(self, object, name):
        """ Disconnect from the Qt signal, if any.
        """
        if self.signal is None:
            # No signal to disconnect from.
            return
        slot_name = _slot_name(name)
        slot = object.__dict__.pop(slot_name, None)
        if slot is not None:
            # Use the classmethod form for PyQt4 compatibility.
            QtCore.QObject.disconnect(object.qobj, self.signal, slot)


class QtProperty(QtTrait):
    """ Proxy trait for a Qt static property.

    Pass in a ``QMetaProperty`` from the ``QMetaObject``.
    """
    def __init__(self, meta_prop, **metadata):
        super(QtProperty, self).__init__(**metadata)
        self.meta_prop = meta_prop
        if meta_prop.hasNotifySignal():
            self.signal = QtCore.SIGNAL(meta_prop.notifySignal().signature())

    def get(self, object, name):
        """ Get the value of this trait.
        """
        qobj = object.qobj
        if qobj is None:
            d = object.__dict__.setdefault(DELAYED_SETATTR, {})
            if name in d:
                return d[name]
            else:
                msg = ("Property {0!r} not available until Binder is given "
                       "its QObject.".format(name))
                raise AttributeError(msg)
        try:
            value = self.meta_prop.read(qobj)
        except RuntimeError:
            # PySide has a bug such that it will not return flags sometimes,
            # like for QGroupBox.alignment.
            name = self.meta_prop.name()
            if hasattr(qobj, name):
                value = getattr(qobj, name)()
            else:
                # Nothing else we can do.
                raise
        # PyQt4 will sometimes return a QPyNullVariant via this API even if it
        # converts it to the correct null value for the type for the property
        # attribute on the QObject itself.
        value = _guard_against_null_variant(value)
        return value

    def set(self, object, name, value):
        """ Set the value of this trait and notify listeners.

        If there is a Qt ``Signal`` for this property, it will notify the
        listeners. If there is not one for this property, this method will
        explicitly send a notification.
        """
        qobj = object.qobj
        if qobj is None:
            d = object.__dict__.setdefault(DELAYED_SETATTR, {})
            d[name] = value
            return
        old = self.get(object, name)
        self.meta_prop.write(qobj, value)
        if self.signal is None:
            # Propagate the event notification ourselves.
            object.trait_property_changed(name, old, value)


class QtDynamicProperty(QtTrait):
    """ A Qt dynamic property added to the ``QObject``.

    The dynamic property will be created on the ``QObject`` when it is
    added to the :class:`~.Binder`. The default value given to this trait will
    be the initial value. It should be an object that can be passed to
    ``QVariant``.

    Because most dynamic properties will be added this way to support Qt
    stylesheets, by default when the property is assigned a new value, the
    ``QObject`` associated with the ``Binder`` (which should be a ``QWidget``)
    will be made to redraw itself in order to reevaluate the stylesheet rules
    with the new value. Turn this off by passing ``styled=False`` to the
    constructor.
    """

    def __init__(self, default_value=None, **metadata):
        metadata['is_dynamic_property'] = True
        metadata.setdefault('styled', True)
        super(QtDynamicProperty, self).__init__(
            default_value=default_value, **metadata)

    def get(self, object, name):
        """ Get the value of this trait.
        """
        qobj = object.qobj
        if qobj is None:
            delayed_attrs = object.__dict__.get(DELAYED_SETATTR, {})
            return delayed_attrs.get(name, self.default_value)
        return qobj.property(name)

    def set(self, object, name, value):
        """ Set the value of this trait and notify listeners.
        """
        qobj = object.qobj
        if qobj is None:
            d = object.__dict__.setdefault(DELAYED_SETATTR, {})
            d[name] = value
            return
        old = qobj.property(name)
        qobj.setProperty(name, value)
        if self.metadata.get('styled', True) and hasattr(qobj, 'style'):
            style = qobj.style()
            style.unpolish(qobj)
            style.polish(qobj)
        object.trait_property_changed(name, old, value)


class QtGetterSetter(QtTrait):
    """ Proxy for a getter/setter pair of methods.

    This is used for ``value()/setValue()`` pairs of methods that are
    frequently found in Qt, but which are not bona fide Qt properties.

    If the names follow this convention, you only need to pass the name of the
    getter method. Otherwise, pass both.
    """
    def __init__(self, getter_name, setter_name=None, **metadata):
        super(QtGetterSetter, self).__init__(**metadata)
        self.getter_name = getter_name
        if setter_name is None:
            setter_name = 'set' + getter_name.title()
        self.setter_name = setter_name

    def get(self, object, name):
        """ Get the value of this trait.
        """
        qobj = object.qobj
        if qobj is None:
            d = object.__dict__.setdefault(DELAYED_SETATTR, {})
            if name in d:
                return d[name]
            else:
                msg = ("Getter {0!r} not available until Binder is given "
                       "its QObject.".format(name))
                raise AttributeError(msg)
        return getattr(qobj, _python_name_for_qt_name(self.getter_name))()

    def set(self, object, name, value):
        """ Set the value of this trait and notify listeners.
        """
        qobj = object.qobj
        if qobj is None:
            d = object.__dict__.setdefault(DELAYED_SETATTR, {})
            d[name] = value
            return
        old = self.get(object, name)
        getattr(qobj, _python_name_for_qt_name(self.setter_name))(value)
        object.trait_property_changed(name, old, value)


class QtSlot(QtTrait):
    """ Proxy for a Qt slot method.

    In general use, this trait will only be assigned to. If the slot takes no
    arguments, the value assigned is ignored. If the slot takes one argument,
    the value assigned is passed to the slot. If the slot takes more than one
    argument, the value assigned should be a tuple of the right size.

    As a convenience, getting the value of this trait will return the slot
    method object itself to allow you to connect to it using the normal Qt
    mechanism.

    The constructor should be passed the ``QMetaMethod`` for this slot.
    """
    def __init__(self, meta_method, **metadata):
        super(QtSlot, self).__init__(**metadata)
        self.meta_method = meta_method
        self.qname = meta_method.signature().split('(')[0]
        self.n_args = len(meta_method.parameterTypes())

    def get(self, object, name):
        """ Get the underlying method object.
        """
        return getattr(object.qobj, _python_name_for_qt_name(self.qname))

    def set(self, object, name, value):
        """ Set the value of this trait.

        See :class:`~.QtSlot` for details on how the value is processed.
        """
        qobj = object.qobj
        if qobj is None:
            d = object.__dict__.setdefault(DELAYED_SETATTR, {})
            d[name] = value
            return
        args = self._process_args(value)
        getattr(qobj, _python_name_for_qt_name(self.qname))(*args)

    def _process_args(self, value):
        if self.n_args == 0:
            # The value is ignored. Setting any value counts as just calling
            # the empty slot.
            args = ()
        elif self.n_args == 1:
            args = (value,)
        else:
            args = value
        return args


class QtSignal(QtSlot):
    """ Proxy for a Qt signal method.

    In general use, this trait will only be listened to for events that are
    emitted internally from Qt. However, it can be assigned values, with the
    same argument semantics as :class:`~.QtSlot`. Like :class:`~.QtSlot`,
    getting the value of this trait will return the signal method object itself
    for you to connect to it using the normal Qt mechanism.

    The constructor should be passed the ``QMetaMethod`` for this signal.
    """
    def __init__(self, meta_method, **metadata):
        super(QtSignal, self).__init__(meta_method, **metadata)
        self.signal = QtCore.SIGNAL(meta_method.signature())

    def set(self, object, name, value):
        """ Emit the signal with the given value.

        See :class:`~.QtSlot` for details on how the value is processed.
        """
        qobj = object.qobj
        if qobj is None:
            d = object.__dict__.setdefault(DELAYED_SETATTR, {})
            d[name] = value
            return
        args = self._process_args(value)
        if len(args) == 0:
            # Use the QMetaMethod to invoke the signal for PyQt4 compatibility.
            self.meta_method.invoke(qobj)
        else:
            # In both PyQt4 and PySide, QMetaMethod.invoke() does not
            # automatically convert the arguments, so emit it directly.
            getattr(qobj, name).emit(*args)


class Rename(object):
    """ Specify that an automatic QtTrait be renamed.

    Use at the class level of a :class:`~.Binder` to rename the trait to
    something else.

    For :class:`~.QtSlot` traits with multiple signatures, only the primary
    part of the name (without the mangled type signature) needs to be given.

    Since one cannot use both a :class:`~.Default` and :class:`~.Rename` at the
    same time, one can also specify the default value here.
    """

    def __init__(self, qt_name, default=Undefined):
        """ Rename an automatic QtTrait.

        Parameters
        ----------
        qt_name : str
            The name of the Qt property/signal/slot being renamed.
        default : object, optional
            The default value, if any.
        """
        self.qt_name = qt_name
        self.default = default

    def __repr__(self):
        extra = ''
        if self.default is not Undefined:
            extra = ', {0.default!r}'.format(self)
        return '{0.__name__}({1.qt_name!r}{2})'.format(type(self), self, extra)


class Default(object):
    """ Specify a default value for an automatic QtTrait.
    """

    def __init__(self, value):
        self.value = value

    def __repr__(self):
        return '{0.__name__}({1.value!r})'.format(type(self), self)


class Binder(HasStrictTraits):
    """ Traited proxy for a ``QObject`` class.

    The default proxy traits will be automatically assigned by inspecting the
    Qt class specified in the :attr:`qclass` class attribute. Since this
    inspection process can be time consuming, compared to normal class
    construction, this will only be done the first time the ``Binder`` class is
    instantiated.

    For those traits that proxy a Qt ``Signal`` (or property that has
    a ``Signal``), the Qt signal connection will only be made once a **Traits**
    listener is attached to the proxy trait.

    The :attr:`qobj` can only be assigned once in the ``Binder's`` lifetime.
    """

    #: The ``QObject`` **class** that is going to be wrapped by this class.
    qclass = QtCore.QObject

    #: The Qt object instance that is wrapped by the :class:`~.Binder`
    #: instance.
    qobj = Instance(QtCore.QObject)

    #: The loopback guard.
    loopback_guard = Instance(LoopbackGuard, args=())

    #: An ID string, if any. It should be a valid Python identifier.
    id = Str()

    def __init__(self, *args, **traits):
        self._initialize_binder_class()
        # HasStrictTraits.__init__ doesn't take *args.
        super(Binder, self).__init__(**traits)

    def construct(self, *args, **kwds):
        """ Default constructor that will automatically instantiate ``qclass``.
        """
        self.qobj = self.qclass(*args, **kwds)

    def configure(self):
        """ Do any configuration of the ``qobj`` that is needed.
        """
        pass

    def dispose(self):
        """ Remove any connections and otherwise clean up for disposal.

        This does not mark any Qt objects for deletion.
        """
        # Remove all signal connections.
        for name, ctrait in self.traits().items():
            if ctrait.is_trait_type(QtTrait):
                ctrait.trait_type.disconnect_signal(self, name)

    def __iter__(self):
        yield self

    def __repr__(self):
        if self.id:
            args = 'id={0.id!r}'.format(self)
        else:
            args = ''
        return '{0.__name__}({1})'.format(type(self), args)

    #### Private protocol #####################################################

    def _on_trait_change(self, handler, name=None, remove=False,
                         dispatch='same', priority=False, target=None):
        """Causes the object to invoke a handler whenever a trait attribute
        is modified, or removes the association.
        """
        check_this_after = False
        n_notifiers_before = 0
        if isinstance(name, six.string_types) and name != 'anytrait':
            # This was attaching a handler to a single trait.
            force_trait = (FORCE_INSTANCE_TRAIT,
                           EXISTING_INSTANCE_TRAIT)[remove]
            force_notifiers = (FORCE_NOTIFIERS,
                               EXISTING_NOTIFIERS)[remove]
            ctrait = self._trait(name, force_trait)
            if ctrait is not None:
                if ctrait.is_trait_type(QtTrait):
                    check_this_after = True
                    notifiers = ctrait._notifiers(force_notifiers)
                    if notifiers:
                        n_notifiers_before = len(notifiers)
        super(Binder, self)._on_trait_change(
            handler, name=name, remove=remove, dispatch=dispatch,
            priority=priority, target=target)
        if check_this_after:
            notifiers = ctrait._notifiers(force_notifiers)
            if notifiers:
                n_notifiers_after = len(notifiers)
            else:
                n_notifiers_after = 0

            connectors = []
            if not n_notifiers_before and n_notifiers_after:
                connectors.append((ctrait.trait_type.connect_signal, name))
            elif n_notifiers_before and not n_notifiers_after:
                connectors.append((ctrait.trait_type.disconnect_signal, name))

            if self.__dict__.get('qobj', None) is None:
                # No Qt object yet. Delay the connection.
                self.__dict__.setdefault(DELAYED_CONNECTION, deque()).extend(
                    connectors)
            else:
                for func, name in connectors:
                    func(self, name)

    def _initialize_binder_class(self):
        """ Ensure that the binder class has been initialized.
        """
        binder_class = self.__class__
        initialized_name = '_{0.__name__}__binder_class_initialized'.format(
            binder_class)
        # Look directly at the __dict__. Even with the name disambiguation, we
        # still only want to look directly at this class, not its superclasses.
        initialized = binder_class.__dict__.get(initialized_name, False)
        if not initialized:
            renamings = {}
            # Find all requested renamings.
            for name in dir(binder_class):
                obj = getattr(binder_class, name)
                if isinstance(obj, Rename):
                    renamings[obj.qt_name] = name
            qt_class = binder_class.qclass
            meta_object = qt_class.staticMetaObject
            seen = set(binder_class.class_trait_names())
            for i in range(meta_object.propertyCount()):
                meta_prop = meta_object.property(i)
                qname = _python_name_for_qt_name(meta_prop.name())
                name = renamings.get(qname, qname)
                if name not in seen:
                    if name.endswith('_'):
                        # See #21
                        continue
                    binder_class.add_class_trait(name, QtProperty(meta_prop))
                    seen.add(name)

            # Ignore any from the superclass or explicitly defined.
            method_names = []
            method_name_counts = Counter()
            for i in range(meta_object.methodCount()):
                meta_meth = meta_object.method(i)
                qname = _python_name_for_qt_name(
                    meta_meth.signature().split('(')[0])
                name = renamings.get(qname, qname)
                method_names.append(name)
                method_name_counts[name] += 1
            for i in range(meta_object.methodCount()):
                meta_meth = meta_object.method(i)
                qname = _python_name_for_qt_name(
                    meta_meth.signature().split('(')[0])
                name = renamings.get(qname, qname)
                if method_name_counts[name] > 1:
                    # Add the argument types to the name to disambiguate.
                    arg_types = [str(t).rstrip('*')
                                 for t in meta_meth.parameterTypes()]
                    name = '_'.join([name] + arg_types)
                if name not in seen:
                    if name.endswith('_'):
                        # See #21
                        continue
                    method_type = meta_meth.methodType()
                    if method_type == meta_meth.Signal:
                        trait = QtSignal(meta_meth)
                    elif method_type == meta_meth.Slot:
                        trait = QtSlot(meta_meth)
                    else:
                        continue
                    binder_class.add_class_trait(name, trait)
                    seen.add(name)

            # Add all getter/setter pairs that we can identify.
            methods = set()
            for name, class_attr in vars(qt_class).items():
                # sip methoddescriptor objects do not have __call__() defined.
                if (callable(class_attr) or
                        type(class_attr).__name__ == 'methoddescriptor'):
                    methods.add(name)
            methods.difference_update(seen)
            for qname in methods:
                # FIXME: We do not know if these are true getter/setters, where
                # the getter has 0 arguments and the setter has just the 1. For
                # example, `QObject.property(name)` and
                # `QObject.setProperty(name, value)` get misidentified here.
                # Unfortunately, the method objects do not have any information
                # about their argument structure.
                putative_setter = 'set' + qname.title()
                if putative_setter in methods:
                    name = renamings.get(qname, qname)
                    if name.endswith('_'):
                        # See #21
                        continue
                    binder_class.add_class_trait(
                        name, QtGetterSetter(qname, putative_setter))
            setattr(binder_class, initialized_name, True)

    def _qobj_changed(self, old, new):
        """ Hook up any delayed connections to the new ``qobj``.
        """
        assert old is None, ("A Binder should only have one QObject per "
                             "lifetime")
        assert new is not None
        # Hook up any delayed connections causd by Traits listeners that were
        # attached before we had a `qobj`.
        if DELAYED_CONNECTION in self.__dict__:
            delayed_connections = self.__dict__.pop(DELAYED_CONNECTION)
            while delayed_connections:
                func, name = delayed_connections.popleft()
                func(self, name)
        # Create and initialized any Qt dynamic properties that have been
        # requested.
        values = {}
        for name, ctrait in self.traits(is_dynamic_property=True).items():
            values[name] = ctrait.trait_type.get_default_value()[1]
        # And any explicit Defaults.
        cls = type(self)
        for name in dir(cls):
            obj = getattr(cls, name)
            if isinstance(obj, Default):
                values[name] = obj.value
            if isinstance(obj, Rename) and obj.default is not Undefined:
                values[name] = obj.default
        # And any delayed setattrs.
        values.update(self.__dict__.pop(DELAYED_SETATTR, {}))
        self.trait_set(**values)


class Composite(Binder):
    """ Base class for Binders that hold other Binders as children.

    Their ``QObjects`` may or may not have a similar parent-child relationship.
    The ``Composite`` is responsible for constructing its children, configuring
    them, and disposing of them.
    """

    #: The child ``Binder`` instances. This will typically be a Property
    #: returning a list of ``Binders`` that are attributes.
    child_binders = Property(List(Instance(Binder)))

    def configure(self):
        """ Do any configuration of the ``qobj`` that is needed.
        """
        for child in self.child_binders:
            child.configure()
        super(Composite, self).configure()

    def dispose(self):
        """ Remove any connections and otherwise clean up for disposal.

        This does not mark any Qt objects for deletion.
        """
        for child in self.child_binders:
            child.dispose()
        super(Composite, self).dispose()

    def __iter__(self):
        yield self
        for child in self.child_binders:
            for x in child:
                yield x

    def _get_child_binders(self):
        """ Default implementation yielding all of the attributes on this
        object that are ``Binders``.
        """
        children = []
        for name, obj in self.__dict__.items():
            if isinstance(obj, Binder):
                # FIXME: uniquify
                children.append(obj)
        return children


class NChildren(Composite):
    """ Base class for Composite Binders that have arbitrary unnamed children.
    """

    #: Any children. It will be filtered for Binders.
    child_binders = List(Instance(Binder))

    def __repr__(self):
        args = ', '.join(map(repr, self.child_binders))
        if self.id:
            args += ', id={0.id!r}'.format(self)
        return '{0.__name__}({1})'.format(type(self), args)



#------------------------------------------------------------------------------
#
#  Copyright (c) 2014-2015, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#------------------------------------------------------------------------------

import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))  # noqa

from traits.api import HasTraits, List, Any, NO_COMPARE
from traitsui.api import View

from qt_binder.raw_widgets import FormLayout, Label
from qt_binder.widgets import EnumDropDown
from qt_binder.bound_editor import Bound


class Dummy(object):
    def __init__(self, name):
        self.name = name

    def __str__(self):
        return self.name

    def __repr__(self):
        return '{0}({1.name!r})'.format(type(self), self)

    def __eq__(self, other):
        if not isinstance(other, type(self)):
            return False
        else:
            return self.name == other.name


class Model(HasTraits):

    #: The current list of (object, label) pairs to display in the combobox.
    values = List()

    #: The alternate lists of values.
    alternates = List([
        ([(Dummy('A'), u'A'),
          (Dummy('B'), u'B'),
          (Dummy('B'), u'Second B')],
         u'First List'),
        ([(Dummy('C'), u'C'),
          (Dummy('A'), u'A'),
          (Dummy('D'), u'D')],
         u'Second List'),
    ])

    #: The value selected.
    value = Any(comparison_mode=NO_COMPARE)

    def default_traits_view(self):
        traits_view = View(
            Bound(
                FormLayout(
                    (u'Which List?', EnumDropDown(id='which_list')),
                    (u'Select:', EnumDropDown(id='select')),
                    (u'Value:', Label(id='value_label')),
                ),
                'which_list.values << object.alternates',
                'which_list.value := object.values',
                'select.values << object.values',
                'select.value := object.value',
                'value_label.text << str(object.value)',
            ),
            resizable=True,
        )
        return traits_view

if __name__ == '__main__':
    m = Model()
    m.values = m.alternates[0][0]
    m.value = Dummy('A')
    m.configure_traits()



import re

from sectiondoc.sections import (
    attributes, methods_table, notes_paragraph, item_list, arguments)
from sectiondoc.renderers import Attribute, Method, Argument, ListItem
from sectiondoc.items import Item, MethodItem
from sectiondoc.items.regex import header_regex
from sectiondoc.styles.doc_render import DocRender
from sectiondoc.styles.style import Style
from sectiondoc.util import add_indent, trim_indent


definition_regex = re.compile(r"""
\*{0,2}            #  no, one or two stars
\w+\s:             #  a word followed by a space and a semicolumn
(.+)?              #  a definition
$                  # match at the end of the line
""", re.VERBOSE)


class PlainDefinition(Item):
    """ A Definition parser that treats anything after the ``:`` as a single
    classifier

    The expected format is::

    +-------------------------------------------------+
    | term [ " : " classifier ]                       |
    +-------------------------------------------------+

    """

    @classmethod
    def is_item(cls, line):
        return definition_regex.match(line) is not None

    @classmethod
    def parse(cls, lines):
        header = lines[0].strip()
        term, classifier = header_regex.split(header, maxsplit=1) if \
            (' :' in header) else (header, '')
        trimed_lines = trim_indent(lines[1:]) if (len(lines) > 1) else ['']
        definition = [line.rstrip() for line in trimed_lines]
        return cls(term.strip(), [classifier.strip()], definition)


def example_paragraph(doc, header, renderer=None, item_class=None):
    """ Render the example section to use the ``::`` directive.

    The section is expected to be given as a paragraph.

    """
    paragraph = doc.get_next_paragraph()
    lines = ['**{0}**'.format(header), '::', '']
    lines += add_indent(paragraph)
    return lines


def returns_section(doc, header, renderer=None, item_class=None):
    """ A return item list section that also accepts a paragraph as
    a description.

    """
    lines = item_list(doc, header, renderer, item_class)
    if len(lines) == 1:
        paragraph = doc.get_next_paragraph()
        lines += add_indent(paragraph)
    return lines


def class_section(lines):
    return DocRender(
        lines,
        sections={
            'Attributes': (attributes, Attribute, PlainDefinition),
            'Arguments': (arguments, Argument, PlainDefinition),
            'Parameters': (arguments, Argument, PlainDefinition),
            'Methods': (methods_table, Method, MethodItem),
            'Note': (notes_paragraph, None, None),
            'Notes': (notes_paragraph, None, None),
            'Example': (example_paragraph, None, None),
            'Examples': (example_paragraph, None, None)})


def function_section(lines):
    return DocRender(
        lines,
        sections={
            'Returns': (returns_section, ListItem, PlainDefinition),
            'Arguments': (arguments, Argument, PlainDefinition),
            'Parameters': (arguments, Argument, PlainDefinition),
            'Raises': (returns_section, ListItem, PlainDefinition),
            'Yields': (item_list, ListItem, PlainDefinition),
            'Note': (notes_paragraph, None, None),
            'Notes': (notes_paragraph, None, None),
            'Example': (example_paragraph, None, None),
            'Examples': (example_paragraph, None, None)})


def setup(app):
    """ Traits custom sectiondoc Style """
    style = Style({
        'class': class_section,
        'function': function_section,
        'method': function_section})
    app.setup_extension('sphinx.ext.autodoc')
    app.connect('autodoc-process-docstring', style.render_docstring)



#------------------------------------------------------------------------------
#
#  Copyright (c) 2014-2015, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#------------------------------------------------------------------------------

import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))  # noqa

from traits.api import HasStrictTraits, Range, Unicode
from traitsui.api import View

from qt_binder.bound_editor import Bound
from qt_binder.qt import QtGui
from qt_binder.widgets import RangeSlider, TextField, UIFile


def localfile(filename):
    return os.path.join(os.path.dirname(__file__), filename)


def format_deg(x):
    return u'{0}\N{DEGREE SIGN}'.format((x + 180) % 360)


class Model(HasStrictTraits):

    text = Unicode(u'123')
    slider_value = Range(-10, 10)
    degrees = Range(0, 359)

    def default_traits_view(self):
        traits_view = View(
            Bound(
                UIFile(
                    localfile('form.ui'),
                    overrides=dict(
                        lineEdit=TextField(validator=QtGui.QIntValidator()),
                    ),
                    insertions=dict(
                        slider=RangeSlider(range=(-10, 10)),
                    ),
                ),
                'lineEdit.value := object.text',
                'dial.value := object.degrees',
                'dial_value.text << format_deg(object.degrees)',
                'slider.value := object.slider_value',
                extra_context=dict(
                    format_deg=format_deg,
                ),
            ),
            resizable=True,
        )
        return traits_view


if __name__ == '__main__':
    m = Model()
    m.configure_traits()



#------------------------------------------------------------------------------
#
#  Copyright (c) 2014-2015, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#------------------------------------------------------------------------------
""" Three types of sliders, integer, float, and logarithmic.
"""

import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))  # noqa

from traits.api import HasTraits, Float, Int
from traitsui.api import View

from qt_binder.api import Bound
from qt_binder.qt import QtGui
from qt_binder.raw_widgets import FormLayout, HBoxLayout, Label
from qt_binder.widgets import IntSlider, FloatSlider, LogSlider


class Model(HasTraits):
    int_value = Int(0)
    float_value = Float(0.0)
    log_value = Float(1.0)

    def default_traits_view(self):
        traits_view = View(
            Bound(
                FormLayout(
                    (u'Integer:', HBoxLayout(
                        IntSlider(id='int_slider', range=(-100, 100)),
                        Label(id='int_text', minimumWidth=50))),
                    (u'Float:', HBoxLayout(
                        FloatSlider(id='float_slider', range=(-10.0, 10.0)),
                        Label(id='float_text', minimumWidth=50))),
                    (u'Log:', HBoxLayout(
                        LogSlider(id='log_slider', range=(0.5, 2.0)),
                        Label(id='log_text', minimumWidth=50))),
                    fieldGrowthPolicy=QtGui.QFormLayout.ExpandingFieldsGrow,
                ),
                'int_slider.value := object.int_value',
                'int_text.text << str(object.int_value)',
                'float_slider.value := object.float_value',
                'float_text.text << u"{:0.2f}".format(object.float_value)',
                'log_slider.value := object.log_value',
                'log_text.text << u"{:0.4f}".format(object.log_value)',
            ),
            resizable=True,
            title=u'Sliders',
        )
        return traits_view


def main():
    m = Model()
    m.configure_traits()

if __name__ == '__main__':
    main()



#------------------------------------------------------------------------------
#
#  Copyright (c) 2015, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#------------------------------------------------------------------------------
import sys


def mock_modules():
    """ Mock missing modules if necessary """

    MOCK_MODULES = []
    MOCK_TYPES = []

    try:
        from qt_binder import qt
    except ImportError:
        MOCK_MODULES = [
            'qt_binder.qt',
            'qt_binder.qt.QtCore',
            'qt_binder.qt.QtGui',
            'qt_binder.qt.qt_api',
            'qt_binder.qt.ui_loader',
            'pyface.qt']
        MOCK_TYPES = []
    else:
        del qt

    TYPES = {
        mock_type: type(mock_type, bases, {'__module__': path})
        for path, mock_type, bases in MOCK_TYPES}

    class DocMock(object):

        def __init__(self, *args, **kwds):
            if '__doc_mocked_name__' in kwds:
                self.__docmock_name__ = kwds['__docmocked_name__']
            else:
                self.__docmock_name__ = 'Unknown'

        def __getattr__(self, name):
            if name in ('__file__', '__path__'):
                return '/dev/null'
            elif name == '__all__':
                return []
            else:
                return TYPES.get(name, DocMock(__docmock_name__=name))

        def __call__(self, *args, **kwards):
            return DocMock()

        def __iter__(self):
            return self

        def __next__(self):
            raise StopIteration()

        def next(self):
            raise StopIteration()

        @property
        def __name__(self):
            return self.__docmock_name__

        def __repr__(self):
            return '<DocMock.{}>'.format(self.__name__)

    sys.modules.update(
        (mod_name, DocMock(mocked_name=mod_name)) for mod_name in MOCK_MODULES)
    print('mocking modules {} and types {}'.format(MOCK_MODULES, MOCK_TYPES))



#------------------------------------------------------------------------------
#
#  Copyright (c) 2014-2015, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#------------------------------------------------------------------------------

from __future__ import division

from math import exp, log
import operator

import six

from traits.api import Any, Callable, Constant, Dict, Enum, Float, Instance, \
    Int, List, NO_COMPARE, Str, Tuple, Undefined, Unicode, on_trait_change

from .binder import Binder, QtDynamicProperty, Rename, Default
from .qt import QtCore, QtGui
from .qt.ui_loader import load_ui
from .raw_widgets import ComboBox, Composite, LineEdit, Slider, binder_registry


INVALID_STYLE_RULE = ("*[valid='false'] "
                      "{ background-color: rgb(255, 192, 192); }")


class TextField(LineEdit):
    """ Simple customization of a LineEdit.

    The widget can be configured to update the model on every text change or
    only when Enter is pressed (or focus leaves). This emulates Traits UI's
    `TextEditor` `auto_set` and `enter_set` configurations.

    If a validator is set, invalid text will cause the background to be red.
    """

    #: The value to sync with the model.
    value = Unicode(comparison_mode=NO_COMPARE)

    #: Whether the `value` updates on every keypress, or when Enter is pressed
    #: (or `focusOut`).
    mode = Enum('auto', 'enter')

    #: Whether or not the current value is valid, for the stylesheet.
    valid = QtDynamicProperty(True)

    def configure(self):
        self.styleSheet = INVALID_STYLE_RULE

    def _update_valid(self, text):
        """ Update the valid trait based on validation of ``text``.
        """
        validator = self.validator
        if validator is not None:
            state, fixed, pos = validator.validate(text, len(text))
            self.valid = (state == validator.Acceptable)

    @on_trait_change('textEdited')
    def _on_textEdited(self, text):
        if (self.mode == 'auto' and
                'value' not in self.loopback_guard):
            with self.loopback_guard('value'):
                self._update_valid(text)
                self.value = text

    @on_trait_change('editingFinished')
    def _on_editingFinished(self):
        if 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                self.value = self.text

    @on_trait_change('text,validator')
    def _on_text(self):
        self._update_valid(self.text)

    def _value_changed(self, new):
        if 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                self.text = new


class EditableComboBox(ComboBox):
    """ ComboBox with an editable text field.

    We do not do bidirectional synchronization of the value with the model
    since that is typically not required for these use cases.
    """

    lineEdit_class = TextField

    #: The selected value.
    value = Any(Undefined, comparison_mode=NO_COMPARE)

    #: (object, label) pairs.
    values = List(Tuple(Any, Unicode))

    #: Function that is used to compare two objects in the values list for
    #: equality. Defaults to normal Python equality.
    same_as = Callable(operator.eq)

    editable = Constant(True)

    def configure(self):
        self.qobj.setEditable(True)
        self._on_editable()
        super(EditableComboBox, self).configure()

    @on_trait_change('values,values_items,qobj')
    def _update_values(self):
        qobj = self.qobj
        if qobj is not None:
            old_value = self.value
            current_text = qobj.currentText()
            current_index = qobj.currentIndex()
            # Check if the user entered in custom text that should be
            # preserved.
            preserve_text = (current_index == -1 or
                             qobj.itemData(current_index) is None or
                             current_text != qobj.itemText(current_index))
            labels = []
            new_index = -1
            for i, (value, label) in enumerate(self.values):
                if self.same_as(value, old_value):
                    new_index = i
                labels.append(label)

            with self.loopback_guard('value'):
                if qobj.count() > 0:
                    qobj.clear()
                # Items from the list get their index into the values list
                # added as their user data as well. Items added from the text
                # field will have that still be None.
                for i, label in enumerate(labels):
                    qobj.addItem(label, i)
            if preserve_text:
                qobj.setEditText(current_text)
                self.value = current_text
            else:
                qobj.setCurrentIndex(new_index)

    @on_trait_change('currentIndexChanged_int')
    def _on_currentIndexChanged(self, index):
        if index != -1 and 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                values_index = self.qobj.itemData(index)
                if values_index is not None:
                    self.value = self.values[values_index][0]
                else:
                    # Otherwise, it's one of the added values.
                    self.value = self.qobj.itemText(index)

    @on_trait_change('lineEdit:textEdited')
    def _on_textEdited(self, text):
        if 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                self.value = text


class EnumDropDown(ComboBox):
    """ Select from a set of preloaded choices.
    """

    #: The selected value.
    value = Any(Undefined, comparison_mode=NO_COMPARE)

    #: (object, label) pairs.
    values = List(Tuple(Any, Unicode))

    #: Function that is used to compare two objects in the values list for
    #: equality. Defaults to normal Python equality.
    same_as = Callable(operator.eq)

    editable = Constant(False)

    @on_trait_change('values,values_items,qobj')
    def _update_values(self):
        qobj = self.qobj
        if qobj is not None:
            old_value = self.value
            labels = []
            if self.editable:
                new_index = -1
            else:
                new_index = 0
            for i, (value, label) in enumerate(self.values):
                if self.same_as(value, old_value):
                    new_index = i
                labels.append(label)

            if qobj.count() > 0:
                qobj.clear()
            qobj.addItems(labels)
            qobj.setCurrentIndex(new_index)

    @on_trait_change('currentIndexChanged_int')
    def _on_currentIndexChanged(self, index):
        if 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                self.value = self.values[index][0]

    def _value_changed(self, new):
        if 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                new_index = -1
                for i, (value, label) in enumerate(self.values):
                    if self.same_as(value, new):
                        new_index = i
                        break
                self.currentIndex = new_index


class UIFile(Composite):
    """ Load a layout from a Qt Designer `.ui` file.

    Widgets and layouts with names that do not start with underscores will be
    added as traits to this :class:`~.Binder`. The :data:`~.binder_registry`
    will be consulted to find the raw :class:`~.Binder` to use for each widget.
    This can be overridden for any named widget using the :attr:`overrides`
    trait.

    In case one wants to let the :class:`~.Binder` to own its widget but just
    use the `.ui` file for layout, use the :attr:`insertions` dictionary. The
    named widget should be a plain `QWidget` in the UI laid out as desired. The
    :class:`~.Binder` will create a new widget as the lone child of this
    widget and take up all of its space.
    """

    qclass = QtGui.QWidget

    #: The .ui file with the layout.
    filename = Str()

    #: Override binders for named widgets.
    overrides = Dict(Str, Instance(Binder))

    #: Insert binders as children of the named QWidgets.
    insertions = Dict(Str, Instance(Binder))

    def __init__(self, filename, **traits):
        super(UIFile, self).__init__(filename=filename, **traits)

    def construct(self, *args, **kwds):
        qobj, to_be_bound = load_ui(self.filename)
        for name in to_be_bound:
            obj = qobj.findChild(QtCore.QObject, name)
            self.add_trait(name, Instance(Binder))
            if name in self.overrides:
                binder = self.overrides[name]
                binder.qobj = obj
            elif name in self.insertions:
                binder = self.insertions[name]
                binder.construct()
                old_layout = obj.layout()
                if old_layout is not None:
                    # Qt hack to replace the layout. We need to ensure that the
                    # old one is truly deleted. Reparent it onto a widget that
                    # we then discard.
                    QtGui.QWidget().setLayout(old_layout)
                layout = QtGui.QVBoxLayout(obj)
                layout.setContentsMargins(0, 0, 0, 0)
                layout.addWidget(binder.qobj)
            else:
                binder = binder_registry.lookup(obj)()
                binder.qobj = obj
            setattr(self, name, binder)
        self.qobj = qobj


class BaseSlider(Slider):
    """ Base class for the other sliders.

    Mostly for interface-checking and common defaults.
    """

    #: The value to synch with the model.
    value = Any(0)

    #: The inclusive range.
    range = Tuple(Any(0), Any(99))

    #: The underlying Qt value.
    qt_value = Rename('value')

    # The Qt default is vertical for some awful reason.
    orientation = Default(QtCore.Qt.Horizontal)


class IntSlider(BaseSlider):

    #: The value to synch with the model.
    value = Int(0)

    #: The inclusive range.
    range = Tuple(Int(0), Int(99))

    def configure(self):
        # Set the initial values.
        self._range_changed()
        self._value_changed()

    def _range_changed(self):
        if self.qobj is not None:
            self.qobj.setRange(*self.range)
        else:
            minimum, maximum = self.range
            self.trait_set(
                minimum=minimum,
                maximum=maximum,
            )

    def _value_changed(self):
        if 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                self.qt_value = self.value

    @on_trait_change('qt_value')
    def _on_qt_value(self):
        if 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                self.value = self.qt_value


class FloatSlider(BaseSlider):

    #: The value to synch with the model.
    value = Float(0.0)

    #: The inclusive range.
    range = Tuple(Float(0.0), Float(1.0))

    #: The number of steps in the range.
    precision = Int(1000)

    def configure(self):
        # Set the initial values.
        self._precision_changed()
        self._value_changed()

    def _precision_changed(self):
        self.maximum = self.precision

    def _range_changed(self):
        self._value_changed()

    def _value_changed(self):
        if 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                self.qt_value = self._qt_value_from_python(self.value)

    @on_trait_change('qt_value')
    def _on_qt_value(self):
        if 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                self.value = self._python_value_from_qt(self.qt_value)

    def _qt_value_from_python(self, value):
        low, high = self.range
        precision = self.precision
        qt_value = int(round((value - low) * precision / (high - low)))
        qt_value = min(max(qt_value, 0), precision)
        return qt_value

    def _python_value_from_qt(self, qt_value):
        low, high = self.range
        precision = self.precision
        value = qt_value * (high - low) / precision + low
        return value


class LogSlider(FloatSlider):

    #: The inclusive range.
    range = Tuple(Float(1e-2), Float(100.0))

    def _qt_value_from_python(self, value):
        low, high = self.range
        precision = self.precision
        value = max(value, low)
        log_low = log(low)
        log_high = log(high)
        log_value = log(value)
        qt_value = int(round((log_value - log_low) *
                             precision /
                             (log_high - log_low)))
        qt_value = min(max(qt_value, 0), precision)
        return qt_value

    def _python_value_from_qt(self, qt_value):
        low, high = self.range
        precision = self.precision
        log_low = log(low)
        log_high = log(high)
        log_value = qt_value * (log_high - log_low) / precision + log_low
        value = exp(log_value)
        return value


class RangeSlider(Composite):
    """ A slider with labels and a text entry field.

    The root widget is a `QWidget` with a new property
    `binder_class=RangeSlider`. Stylesheets can reference it using the
    selector::

        *[binder_class="RangeSlider"] {...}

    This can be useful for styling the child `QLabels` and `QLineEdit`, for
    example to make a series of `RangeSliders` align.
    """
    qclass = QtGui.QWidget

    #: The value to synch with the model.
    value = Any(0)

    #: The inclusive range.
    range = Tuple(Any(0), Any(99))

    #: The formatting function for the labels.
    label_format_func = Callable(six.text_type)

    #: The formatting function for the text field. This is used only when the
    #: slider is setting the value.
    field_format_func = Callable(six.text_type)

    #: The slider widget.
    slider = Instance(BaseSlider, factory=IntSlider, args=())

    #: The field widget.
    field = Instance(TextField, args=())

    _low_label = Any()
    _high_label = Any()
    _from_text_func = Callable(int)

    def __init__(self, *args, **traits):
        # Make sure that a `slider` argument gets assigned before anything else
        # because it will affect what range can be accepted.
        if 'slider' in traits:
            slider = traits.pop('slider')
            super(RangeSlider, self).__init__()
            self.slider = slider
            self.trait_set(**traits)
        else:
            super(RangeSlider, self).__init__(*args, **traits)

    def construct(self):
        self.slider.construct()
        self.field.construct()
        super(RangeSlider, self).construct()
        self.qobj.setProperty('binder_class', u'RangeSlider')
        layout = QtGui.QHBoxLayout()
        self._low_label = QtGui.QLabel()
        self._low_label.setAlignment(QtCore.Qt.AlignRight)
        self._high_label = QtGui.QLabel()
        self._high_label.setAlignment(QtCore.Qt.AlignLeft)
        layout.addWidget(self._low_label)
        layout.addWidget(self.slider.qobj)
        layout.addWidget(self._high_label)
        layout.addWidget(self.field.qobj)
        layout.setContentsMargins(0, 0, 0, 0)
        self.qobj.setLayout(layout)

    def configure(self):
        super(RangeSlider, self).configure()
        if isinstance(self.slider, IntSlider):
            # Use an integer validator for the text field.
            self.field.validator = QtGui.QIntValidator()
            self._from_text_func = int
        else:
            self.field.validator = QtGui.QDoubleValidator()
            self._from_text_func = float
        self._update_widgets()

    @on_trait_change('value,range')
    def _update_widgets(self):
        # Update the range then the value because the widgets will just
        # silently reject values out of bounds.
        if 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                value = self.value
                range = self.range
                if self.qobj is not None:
                    self.field.validator.setRange(range[0], range[1])
                    if not isinstance(self.slider, IntSlider):
                        # Note: this assumes that all sliders other than
                        # IntSlider have decimal inputs.
                        self.field.validator.setDecimals(16)
                    self._low_label.setText(self.label_format_func(range[0]))
                    self._high_label.setText(self.label_format_func(range[1]))
                self.field.text = six.text_type(value)
                self.slider.range = range
                self.slider.value = value

    @on_trait_change('slider:value')
    def _on_slider_value(self):
        if 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                value = self.slider.value
                self.value = value
                self.field.text = self.field_format_func(value)

    @on_trait_change('field:value')
    def _on_field_text(self, text):
        if 'value' not in self.loopback_guard:
            with self.loopback_guard('value'):
                if self.field.valid:
                    try:
                        value = self._from_text_func(text)
                    except ValueError:
                        pass
                    else:
                        self.value = value
                        self.slider.value = value



from setuptools import setup, find_packages


long_description = """\
QtBinder provides a way to bind Qt widgets to Traits models.
"""

setup(
    name='qt_binder',
    version='0.2',
    author='Enthought, Inc.',
    author_email='info@enthought.com',
    description='Traits bindings for Qt',
    long_description=long_description,
    url='https://github.com/enthought/qt_binder',
    license="BSD",
    packages=find_packages(exclude=['examples', 'examples.*']),
    package_data={
        'qt_binder.tests': ['*.ui'],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
    use_2to3=True,
)



from . import qt_api

if qt_api == 'pyqt':
    def load_ui(path):
        from collections import Counter
        from PyQt4 import uic

        ui = uic.loadUi(path)
        names_hist = Counter()
        for child in ui.children():
            name = child.objectName()
            if name:
                names_hist[name] += 1

        names = []
        for name, count in names_hist.items():
            if count == 1 and not name.startswith('_'):
                names.append(name)

        return ui, names

else:
    def load_ui(path):
        from collections import Counter
        from PySide.QtUiTools import QUiLoader

        class RecordingUiLoader(QUiLoader):
            """ Record the names of widgets as they are created.
            """

            def __init__(self, *args, **kwds):
                self.names = Counter()
                super(RecordingUiLoader, self).__init__(*args, **kwds)

            def to_be_bound(self):
                """ Return the names of child widgets/layouts to be bound.
                """
                names = []
                for name, count in self.names.items():
                    if count == 1 and not name.startswith('_'):
                        names.append(name)
                return names

            def createLayout(self, className, parent=None, name=u''):
                if name:
                    self.names[name] += 1
                layout = super(RecordingUiLoader, self).createLayout(
                    className, parent, name)
                return layout

            def createWidget(self, className, parent=None, name=u''):
                if name:
                    self.names[name] += 1
                widget = super(RecordingUiLoader, self).createWidget(
                    className, parent, name)
                return widget

        loader = RecordingUiLoader()
        ui = loader.load(path)
        names = loader.to_be_bound()
        # Exclude the root object from this list as it is always bound to the
        # UIFile itself.
        self_name = ui.objectName()
        if self_name in names:
            names.remove(self_name)

        return ui, names
